function ita_modulitacontrol_gui(varargin)
% ITA_MODULITACONTROL_GUI - gui for modulITA remote control

% <ITA-Toolbox>
% This file is part of the application RoboAurelioModulITAControl for the ITA-Toolbox. All rights reserved.
% You can find the license for this m-file in the application folder.
% </ITA-Toolbox>

oldSettings = ita_modulita_control('getSettings');
persistent hFigure

%% nice gui settings
gui_bg_color  = [0.8 0.8 0.8];

if nargin == 0
    width  = 400;
    height = 520;
    
    mpos = get(0,'Monitor'); %try to position in the middle of the screen
    w_position = (mpos(1,length(mpos)-1)/2)-(width/2);
    h_position = (mpos(1,length(mpos))/2)-(height/2);
    MainPosition = [w_position h_position width height];
    
    %% Figure Handling
    if isempty(hFigure) || ~ishandle(hFigure)
        hFigure = figure( ...       % the main GUI figure
            'MenuBar', 'none', ...
            'Toolbar', 'none', ...
            'HandleVisibility', 'on', ...
            'Name', 'modulITA Remote Control', ...
            'NumberTitle', 'off', ...
            'Position' , MainPosition, ...
            'Color', gui_bg_color);
    elseif ~strcmpi(get(hFigure,'Name'),'modulITA Remote Control')
        hFigure = figure( ...       % the main GUI figure
            'MenuBar', 'none', ...
            'Toolbar', 'none', ...
            'HandleVisibility', 'on', ...
            'Name', 'ITA-Toolbox Preferences', ...
            'NumberTitle', 'off', ...
            'Position', MainPosition, ...
            'Color', gui_bg_color);
    else %we are in an ITA-Toolbox Preferences Window (pdi)
        clf(hFigure)
    end
    %% ITA toolbox logo
    a_im = importdata('ita_toolbox_logo.jpg');
    image(a_im);axis off
    set(gca,'Units','pixel', 'Position', [20 10 350 65]*0.6);
else %reset mode
    %     clf(hFigure) %clear old settings
end

%%
hor_start = 4;
ver_start = 5;
hor_space = 2;
ver_space = 0.5;

button_height = 1.5;
button_width = 12;

for ch_idx = 1:4
    %% FEED
    userdata.ch = ch_idx;
    
    nameStr = {'off','Pol','Pha','P+P'};
    nButtons = length(nameStr);
    InputFeed{ch_idx}.Size = [button_width+1 (nButtons+1) * (button_height)];
    InputFeed{ch_idx}.Position = [hor_start ver_start InputFeed{ch_idx}.Size];
    InputFeed{ch_idx}.h = uibuttongroup(...
        'Parent',hFigure,...
        'Units','characters',...
        'FontSize',10,...
        'Title','Feed',...
        'Tag','Mode',...
        'BackgroundColor',gui_bg_color,...
        'Clipping','on',...
        'Position',InputFeed{ch_idx}.Position,...
        'SelectedObject',[],...
        'SelectionChangeFcn',@InputFeedCallback,...
        'OldSelectedObject',[]);
    
    position =  [0 0 button_width button_height];
    for button_idx = 1:nButtons
        InputFeed{ch_idx}.hButton(button_idx) = uicontrol(...
            'Parent',InputFeed{ch_idx}.h,...
            'Units','characters',...
            'FontSize',10,...
            'Position',[position],...
            'BackgroundColor',[1 0 0],...
            'String',nameStr{button_idx},...
            'Style','togglebutton',...
            'UserData',userdata,...
            'Value',strcmpi(nameStr{button_idx},oldSettings.ch(ch_idx).inputfeed),...
            'Tag','togglebuttonNorm');
        position = position + [0 button_height 0 0];
    end
    
    %% INPUT SELECT
    nameStr = {'off','Lemo','XLR','Ref'};
    nButtons = length(nameStr);
    InputSelect{ch_idx}.Size = [button_width+1 (nButtons+1) * (button_height)];
    InputSelect{ch_idx}.Position = [InputFeed{ch_idx}.Position(1)  InputFeed{ch_idx}.Position(2)+ InputFeed{ch_idx}.Position(4)+ver_space InputSelect{ch_idx}.Size];
    InputSelect{ch_idx}.h = uibuttongroup(...
        'Parent',hFigure,...
        'Units','characters',...
        'FontSize',10,...
        'Title','Select',...
        'Tag','Mode',...
        'Clipping','on',...
        'BackgroundColor',gui_bg_color,...
        'Position',InputSelect{ch_idx}.Position,...
        'SelectedObject',[],...
        'SelectionChangeFcn',@InputSelectCallback,...
        'OldSelectedObject',[]);
    
    position =  [0 0 button_width button_height];
    for button_idx = 1:nButtons
        InputSelect{ch_idx}.hButton(button_idx) = uicontrol(...
            'Parent',InputSelect{ch_idx}.h,...
            'Units','characters',...
            'FontSize',10,...
            'Position',[position],...
            'String',nameStr{button_idx},...
            'UserData',userdata,...
            'Style','togglebutton',...
            'Value',strcmpi(nameStr{button_idx},oldSettings.ch(ch_idx).inputselect),...
            'Tag','togglebuttonNorm');
        position = position + [0 button_height 0 0];
    end
    
    %% INPUT RANGE
    inputRange_vec = [40:-10:-60];
    nameStr = cellstr(num2str(inputRange_vec'));
    nButtons = length(nameStr);
    
    InputRange{ch_idx}.Size = [button_width+1 (nButtons+1) * (button_height)];
    InputRange{ch_idx}.Position = [InputSelect{ch_idx}.Position(1)  InputSelect{ch_idx}.Position(2)+ InputSelect{ch_idx}.Position(4)+ver_space InputRange{ch_idx}.Size];
    InputRange{ch_idx}.h = uibuttongroup(...
        'Parent',hFigure,...
        'Units','characters',...
        'FontSize',10,...
        'Title',['Range ch' num2str(ch_idx)],...
        'Tag','Mode',...
        'BackgroundColor',gui_bg_color,...
        'Clipping','on',...
        'Position',InputRange{ch_idx}.Position,...
        'SelectedObject',[],...
        'SelectionChangeFcn',@InputRangeCallback,...
        'OldSelectedObject',[]);
    
    position =  [0 0 button_width button_height];
    for button_idx = 1:nButtons
        InputRange{ch_idx}.hButton(button_idx) = uicontrol(...
            'Parent',InputRange{ch_idx}.h,...
            'Units','characters',...
            'FontSize',10,...
            'Position',[position],...
            'String',nameStr{button_idx},...
            'BackgroundColor',[0.1 0.9 0.1],...
            'UserData',userdata,...
            'Style','togglebutton',...
            'Value',oldSettings.ch(ch_idx).inputrange == inputRange_vec(button_idx),...
            'Tag','togglebuttonNorm');
        position = position + [0 button_height 0 0];
    end
    
    hor_start = hor_start + button_width + hor_space;
    
end

%% right column
nameStr = {};
argCell = {};
ele = 1;
nameStr{ele} = {['Norm']};
argCell{ele} = {'mode','norm'};
ele = ele + 1;
nameStr{ele} = {['AmpRef']};
argCell{ele} = {'mode','ampref'};
ele = ele + 1;
nameStr{ele} = {['LineRef']};
argCell{ele} = {'mode','lineref'};
ele = ele + 1;
nameStr{ele} = {['ImpRef']};
argCell{ele} = {'mode','impref'};
ele = ele + 1;
nameStr{ele} = {['LineRef']};
argCell{ele} = {'mode','lineref'};

%% generate modebuttons
InputFeed = {InputFeed{1}};
nButtons = length(nameStr);
InputFeed{1}.Size = [button_width+1 (nButtons+1) * (button_height)];
InputFeed{1}.Position = [hor_start+1 ver_start InputFeed{1}.Size];
InputFeed{1}.h = uibuttongroup(...
    'Parent',hFigure,...
    'Units','characters',...
    'FontSize',10,...
    'Title','Mode',...
    'Tag','Mode',...
    'Clipping','on',...
    'Position',InputFeed{1}.Position,...
    'BackgroundColor',gui_bg_color,...
    'SelectedObject',[],...
    'OldSelectedObject',[]);
ch_idx = 1;
position =  [0 0 button_width button_height];
for button_idx = 1:nButtons
    userdata = argCell{button_idx};
    ControlButton{ch_idx}.hButton(button_idx) = uicontrol(...
        'Parent',InputFeed{ch_idx}.h,...
        'Units','characters',...
        'FontSize',10,...
        'Position',[position],...
        'String',nameStr{button_idx},...
        'Style','pushbutton',...
        'Callback',@modePushButtonCallback,...
        'UserData',userdata,...
        'Value',0,...
        'Tag','');
    position = position + [0 button_height 0 0];
end


%% davolume
nameStr = {};
argCell = {};
ele = 1;
nameStr{ele} = {['DA--']};
argCell{ele} = {'davolume--'};
ele = ele + 1;
nameStr{ele} = {['DA++']};
argCell{ele} = {'davolume++'};
ele = ele + 1;
nameStr{ele} = {['Init']};
argCell{ele} = {'init'};

%% controlbuttons (push)
nButtons = length(nameStr);

InputFeed{1}.Size = [button_width+1 (nButtons+1) * (button_height)];
InputFeed{1}.Position = [InputFeed{1}.Position(1)  InputFeed{1}.Position(2)+ InputFeed{1}.Position(4)+ver_space InputFeed{1}.Size];
%     InputFeed{ch_idx}.Position = [hor_start ver_start InputFeed{ch_idx}.Size];
InputFeed{1}.h = uipanel(...
    'Parent',hFigure,...
    'Units','characters',...
    'FontSize',10,...
    'Title','Control',...
    'Tag','Mode',...
    'Clipping','on',...
    'Position',InputFeed{ch_idx}.Position,...
    'BackgroundColor',gui_bg_color);


position =  [0 0 button_width button_height];
for button_idx = 1:length(nameStr)
    userdata = argCell{button_idx};
    ControlButton{ch_idx}.hButton(button_idx) = uicontrol(...
        'Units','characters',...
        'Parent',InputFeed{1}.h,...
        'FontSize',10,...
        'Position',[position],...
        'String',nameStr{button_idx},...
        'Style','pushbutton',...
        'Callback',@controlPushButtonCallback,...
        'UserData',userdata,...
        'Value',0,...
        'Tag','');
    position = position + [0 button_height 0 0];
end

%% callback functions

    function modePushButtonCallback(h,event)
        userdata = getfield(get(h),'UserData');
        argCell = userdata;
        if ~isempty(argCell)
            ita_modulita_control(argCell{:});
            ita_modulitacontrol_gui('init');
            
        end
        
    end



    function controlPushButtonCallback(h,event)
        argCell = getfield(get(h),'UserData'); %#ok<*GFLD>
        if ~isempty(argCell)
            if findstr('davolume',argCell{1}) %#ok<*REMFF1>
                a = ita_modulita_control('getSettings');
                davolume = a.davolume;
                if findstr ('--',argCell{1})
                    davolume = davolume - 1;
                else
                    davolume = davolume + 1;
                end
                ita_modulita_control('davolume',davolume);
                %                 set(event.NewValue,'UserData',argCell);
            else
                ita_modulita_control(argCell{:});
                ita_modulitacontrol_gui('init');
            end
        end
        
    end



    function InputSelectCallback(h,event)
        userdata = getfield(get(event.NewValue),'UserData');
        inputselect = getfield(get(event.NewValue),'String');
        ita_modulita_control('channel',userdata.ch,'input',inputselect);
    end


    function InputFeedCallback(h,event)
        userdata = getfield(get(event.NewValue),'UserData');
        inputfeed = getfield(get(event.NewValue),'String');
        ita_modulita_control('channel',userdata.ch,'feed',inputfeed);
    end


    function InputRangeCallback(h,event)
        userdata = getfield(get(event.NewValue),'UserData');
        value = getfield(get(event.NewValue),'String');
        inputrange = str2num(value); %#ok<ST2NM>
        ita_modulita_control('channel',userdata.ch,'inputrange',inputrange);
    end

end