function ita_italian_gui(iMS)
%ITA_GENERATE - Generates GUI for ita_italian function
% Makes the turntable move left, right, up or down by steps of 1,5 or 10
% degrees.
%
% Syntax:  ita_italian_gui()
% Syntax:  ita_italian_gui(iMS)
%
% Autor: Alexandra Craciun -- Email: Alexandra.Craciun@rwth-aachen.de
% Created:  19-Mai-2009
%

% <ITA-Toolbox>
% This file is part of the application Movtec for the ITA-Toolbox. All rights reserved. 
% You can find the license for this m-file in the application folder. 
% </ITA-Toolbox>


%% Initialization
degree_range = 3; % how many degree options you want for the gui (e.g 3 for 1,5 and 10 degrees)
top_margin   = 25;
hor_space_s  = 5;
height       = top_margin + (2*degree_range+1) * (hor_space_s + 30);
width        = 800;
width_button = 100;
MainPosition = [100 500 width height];

% get italian setup
if nargin == 0
    iMS = itaItalian;
    iMS.defaultTurntableSpeed = 5;   % default speed for moving the turntable
    iMS.defaultArmSpeed       = 0.5; % default speed for moving the arm
end

if ~iMS.isInitialized
    iMS.init;
end

global move_arm_pos
if isempty(move_arm_pos)
    move_arm_pos = 0;
end
global move_turntable_pos
if isempty(move_turntable_pos)
    move_turntable_pos = 0;
end

%% Main Figure
hMainFigure = figure( ...       % the main GUI figure
    'MenuBar','none', ...
    'Toolbar','none', ...
    'HandleVisibility','on', ...
    'Name', 'Italian Turntable', ...
    'NumberTitle','off', ...
    'Position' , MainPosition, ...
    'Color', [0.8 0.8 0.8]);
figSet.hMainFigure = hMainFigure;

%% Buttons
% Speed
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 - (hor_space_s + width_button)*2 (height-30)/2 + (hor_space_s + 25)*3  width_button 30],...
    'String','Speed',...
    'Style', 'text',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8]);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 - (hor_space_s + width_button)*1 (height-30)/2 + (hor_space_s + 30)*2  width_button 30],...
    'String','Slow',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.4 0.8 0.8],...
    'Callback',@Slow);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 - (hor_space_s + width_button)*2 (height-30)/2 + (hor_space_s + 30)*2  width_button 30],...
    'String','Normal',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.4 0.8 0.8],...
    'Callback',@Normal);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 - (hor_space_s + width_button)*3 (height-30)/2 + (hor_space_s + 30)*2  width_button 30],...
    'String','Fast',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.4 0.8 0.8],...
    'Callback',@Fast);

% Reference movement
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 (height-30)/2 width_button 30],...
    'String','Ref',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.2 0.2],...
    'Callback',@Ref);

% Move arm up
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 (height-30)/2 + (hor_space_s + 30)*1 width_button 30],...
    'String','1',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move1Up);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 (height-30)/2 + (hor_space_s + 30)*2 width_button 30],...
    'String','5',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move5Up);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 (height-30)/2 + (hor_space_s + 30)*3 width_button 30],...
    'String','10',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move10Up);

% Move arm down
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 (height-30)/2 - (hor_space_s + 30)*1 width_button 30],...
    'String','1',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move1Down);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 (height-30)/2 - (hor_space_s + 30)*2 width_button 30],...
    'String','5',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move5Down);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 (height-30)/2 - (hor_space_s + 30)*3 width_button 30],...
    'String','10',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move10Down);

% Move turntable to the right
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*1 (height-30)/2  width_button 30],...
    'String','1',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move1Right);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*2 (height-30)/2  width_button 30],...
    'String','5',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move5Right);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*3 (height-30)/2  width_button 30],...
    'String','10',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move10Right);

% Move turntable to the left
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 - (hor_space_s + width_button)*1 (height-30)/2  width_button 30],...
    'String','1',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move1Left);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 - (hor_space_s + width_button)*2 (height-30)/2  width_button 30],...
    'String','5',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move5Left);

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 - (hor_space_s + width_button)*3 (height-30)/2  width_button 30],...
    'String','10',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Move10Left);

% Cancel button
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*2 (height-200)/2  width_button 30],...
    'String','Cancel',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@Cancel);

% Position
% Turntable position
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*1 (height+130)/2  width_button 30],...
    'String','Turntable Position',...
    'Style', 'text',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8]);

hTurntablePosition = uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*1 (height+100)/2  width_button 25],...
    'String',move_turntable_pos,...
    'Style', 'edit',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [1 1 1]);

%Arm position
uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*3 (height+130)/2  width_button 30],...
    'String','Arm Position',...
    'Style', 'text',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8]);

hArmPosition = uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*3 (height+100)/2  width_button 25],...
    'String',move_arm_pos,...
    'Style', 'edit',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [1 1 1]);

% Update Position

uicontrol(...
    'Parent', hMainFigure,...
    'Position', [(width-width_button)/2 + (hor_space_s + width_button)*2 (height+100)/2  width_button 25],...
    'String','Update Position',...
    'Style', 'pushbutton',...
    'ForegroundColor', [0 0 .7],...
    'BackgroundColor', [0.8 0.8 0.8],...
    'Callback',@UpdatePosition);


%% ita toolbox logo
a_im = importdata(which('ita_toolbox_logo.jpg'));
image(a_im);axis off
set(gca,'Units','pixel', 'Position', [(width-width_button)/2 - (hor_space_s + width_button)*3.3 (height-200)/2 300 50]); %pdi new scaling

%% Buttons
% set speed buttons
    function Slow(hObject,eventdata)
        iMS.defaultTurntableSpeed = 1/12;
        iMS.defaultArmSpeed = 1/12;
    end

    function Normal(hObject,eventdata)
        iMS.defaultTurntableSpeed = 5;
        iMS.defaultArmSpeed = 0.5;
    end

    function Fast(hObject,eventdata)
        iMS.defaultTurntableSpeed = 10;
        iMS.defaultArmSpeed = 1;
    end

% move to reference position button
    function Ref(hObject,eventdata) %#ok<INUSD>
        iMS.referenceMove;
    end

% move up buttons
    function Move1Up(hObject,eventdata) %#ok<INUSD>
        iMS.move_arm(1);
    end

    function Move5Up(hObject,eventdata) %#ok<INUSD>
        iMS.move_arm(5);
    end

    function Move10Up(hObject,eventdata) %#ok<INUSD>
        iMS.move_arm(10);
    end

% move down buttons
    function Move1Down(hObject,eventdata) %#ok<INUSD>
        iMS.move_arm(-1);
    end

    function Move5Down(hObject,eventdata) %#ok<INUSD>
        iMS.move_arm(-5);
    end

    function Move10Down(hObject,eventdata) %#ok<INUSD>
        iMS.move_arm(-10);
    end

% move right buttons
    function Move1Right(hObject,eventdata) %#ok<INUSD>
        iMS.move_turntable(1);
    end

    function Move5Right(hObject,eventdata) %#ok<INUSD>
        iMS.move_turntable(5);
    end

    function Move10Right(hObject,eventdata) %#ok<INUSD>
        iMS.move_turntable(10);
    end

% move left buttons
    function Move1Left(hObject,eventdata) %#ok<INUSD>
        iMS.move_turntable(-1);
    end

    function Move5Left(hObject,eventdata) %#ok<INUSD>
        iMS.move_turntable(-5);
    end

    function Move10Left(hObject,eventdata) %#ok<INUSD>
        iMS.move_turntable(-10);
    end

% update position button
    function UpdatePosition(hObject,eventdata)
        %TODO
        set(hTurntablePosition,'String',ita_italian_get_position(1));
        set(hArmPosition,'String',ita_italian_get_position(2));
    end
% cancel button
    function Cancel(hObject,eventdata) %#ok<INUSD>
        uiresume(gcf);
        close(gcf)
        return;
    end

end % end function