function state = ita_directivityAngles(params,state,dirs)
% Noam Shabtai
% Institution of Technical Acoustics
% RWTH Aachen
% nsh@akustik.rwth-aachen.de
% 24.7.2014

switch params.stages.directivity_angles
case 0
    return;
case 1
    if params.display.headers disp('Setting receiver directivity angles...');end
    source_to_receiver_displacements = state.source_to_receiver_displacements;
    left_image_to_receiver_displacements = state.left_image_to_receiver_displacements;
    bottom_image_to_receiver_displacements = state.bottom_image_to_receiver_displacements;
    backward_image_to_receiver_displacements = state.backward_image_to_receiver_displacements;
    left_bottom_image_to_receiver_displacements = state.left_bottom_image_to_receiver_displacements;
    left_backward_image_to_receiver_displacements = state.left_backward_image_to_receiver_displacements;
    bottom_backward_image_to_receiver_displacements = state.bottom_backward_image_to_receiver_displacements;
    left_bottom_backward_image_to_receiver_displacements = state.left_bottom_backward_image_to_receiver_displacements;

    [receiver_to_source_phi, receiver_to_source_theta] = cart2sph(-source_to_receiver_displacements(:,1),-source_to_receiver_displacements(:,2),-source_to_receiver_displacements(:,3));
    [receiver_to_left_image_phi, receiver_to_left_image_theta] = cart2sph(-left_image_to_receiver_displacements(:,1),-left_image_to_receiver_displacements(:,2),-left_image_to_receiver_displacements(:,3));
    [receiver_to_bottom_image_phi, receiver_to_bottom_image_theta] = cart2sph(-bottom_image_to_receiver_displacements(:,1),-bottom_image_to_receiver_displacements(:,2),-bottom_image_to_receiver_displacements(:,3));
    [receiver_to_backward_image_phi, receiver_to_backward_image_theta] = cart2sph(-backward_image_to_receiver_displacements(:,1),-backward_image_to_receiver_displacements(:,2),-backward_image_to_receiver_displacements(:,3));
    [receiver_to_left_bottom_image_phi, receiver_to_left_bottom_image_theta] = cart2sph(-left_bottom_image_to_receiver_displacements(:,1),-left_bottom_image_to_receiver_displacements(:,2),-left_bottom_image_to_receiver_displacements(:,3));
    [receiver_to_left_backward_image_phi, receiver_to_left_backward_image_theta] = cart2sph(-left_backward_image_to_receiver_displacements(:,1),-left_backward_image_to_receiver_displacements(:,2),-left_backward_image_to_receiver_displacements(:,3));
    [receiver_to_bottom_backward_image_phi, receiver_to_bottom_backward_image_theta] = cart2sph(-bottom_backward_image_to_receiver_displacements(:,1),-bottom_backward_image_to_receiver_displacements(:,2),-bottom_backward_image_to_receiver_displacements(:,3));
    [receiver_to_left_bottom_backward_image_phi, receiver_to_left_bottom_backward_image_theta] = cart2sph(-left_bottom_backward_image_to_receiver_displacements(:,1),-left_bottom_backward_image_to_receiver_displacements(:,2),-left_bottom_backward_image_to_receiver_displacements(:,3));

    directivity_angles.theta_Rx1 = [...
    receiver_to_source_theta;...
    receiver_to_left_image_theta;...
    receiver_to_bottom_image_theta;...
    receiver_to_backward_image_theta;...
    receiver_to_left_bottom_image_theta;...
    receiver_to_left_backward_image_theta;...
    receiver_to_bottom_backward_image_theta;...
    receiver_to_left_bottom_backward_image_theta;...
    ];
    directivity_angles.phi_Rx1 = [...
    receiver_to_source_phi;...
    receiver_to_left_image_phi;...
    receiver_to_bottom_image_phi;...
    receiver_to_backward_image_phi;...
    receiver_to_left_bottom_image_phi;...
    receiver_to_left_backward_image_phi;...
    receiver_to_bottom_backward_image_phi;...
    receiver_to_left_bottom_backward_image_phi;...
    ];

    if params.mode.save_intermediate_results
        save(fullfile(dirs.parent_dir,dirs.mat_dir,...
                      dirs.directivity_angles_filename),...
            'directivity_angles');
    end
case 2
    if params.display.headers disp('Loading image radiation angles...');end
    load(fullfile(dirs.parent_dir,dirs.mat_dir,...
                  dirs.directivity_angles_filename),...
        'directivity_angles');
end

state.directivity_angles = directivity_angles;
