function defined_preferences = ita_defined_preferences()
%% Definition depending whether license file is available or not

% <ITA-Toolbox>
% This file is part of the ITA-Toolbox. Some rights reserved.
% You can find the license for this m-file in the license.txt file in the ITA-Toolbox folder.
% </ITA-Toolbox>

if numel(rdir([ita_toolbox_path filesep 'Licenses' filesep '**' filesep 'License_*.txt']))
    defined_preferences = {...
        'AuthorStr','','LicenseFile','Full Name','Name in license file.',1;... %RSC - this is doubled for backward compatibility, does not matter anyway
        'EmailStr','@akustik.rwth-aachen.de','LicenseFile','Email Address','EMail address in license file.',1;... %RSC - this is doubled for backward compatibility, does not matter anyway
        'Passphrase',repmat('*',1,40),'*LicenseFile','Passphrase','Copy and paste your passphrase/license key from the Toolbox-License email.',1;...
        };
else
    defined_preferences = {...
        'AuthorStr','','char','Full Name','Copy and paste your name from the Toolbox-License email.',1;...
        'EmailStr','@','char','Email Address','Copy and paste your email address from the Toolbox-License email.',1;...
        };
end

%% Definiton for everyone
defined_preferences = [defined_preferences; {...
    %variablename %defaultvalue %type %Name shown in GUI % help text % tabnumber to appear in
    % GENERAL SETTINGS Tab
    'verboseMode',0,'int_verboseMode','Verbosity level','This will show additional information in the command line when running toolbox functions.',1;...
    'freqRange',[20 20000],'int','Frequency Range','Use this frequency range for evaluation of bandfilters, etc.',1;...
    'bandsperoctave',3,'popup_double','Bands per Octave','Bandwidth for fractional octave band filters.[1|3|12|24]',1;...
    'blackbackground',false,'bool','Black background','This changes the color scheme of all MATLAB plots.',1;...
    'menubar',true,'bool','MATLAB menu bar','Do you want a matlab menu bar?',1;...
    'itamenu',true','bool','ITA menu', 'Insert ITA specific menu to all plots',1;...
    'legend',true,'bool','Plot legend', 'Automatic nice legend in all plots',1;...
    'plotcursors',true ,'bool','Cursor plots (slow)','Do you want MF-style cursors to interact in plots?',1;...
    'fontsize',[],'int','Font Size','Font Size for figures',1;...
    'fontname',[],'char','Font Type','Font Type for figures',1;...
    'linewidth',0.5,'double','Linewidth in plots','Linewidth for the ita plot routines',1;...
    'aspectratio',[],'int','Figure aspect ratio','empty is for full screen, any other number will result in a fixed ratio of height and width.',1;...
    'nodb',false,'bool','Linear y-Axis','Linear y-Axis for Frequency plots',1;...
    'linfreq',false,'bool','Linear Freq-Axis','Linear x-Axis for Frequency plots',1;...
    
    % EXPERT STUFF Tab
    'dispVerboseChannels',true,'bool','Display channel information','',3;...
    'dispVerboseHistory',false,'bool','Display history log','',3;...
    'dispVerboseErrors',true,'bool','Display error log','',3;...
    'dispVerboseFunctions',true,'bool','Display useful functions','',3;...
    'eps2pdf_configured',false,'*bool_ispc','Do you have eps2pdf configured?','Only on Windows, will configure eps2pdf once and then be set to true. To reconfigure disable and restart ita_toolbox_setup.',3;...
    'WindowsRegistryEntryChecked',false,'*bool_ispc','Directly open ITA-files with Toolbox?','Only on Windows.',3;... %pdi:problem this is the inverse % TODO %
    'isGraphVizInstalled',0,'bool','GraphViz installed','GraphViz is used to generate nice dependency graphs in the online help files.',3;...
    'isGhostscriptInstalled',0,'bool','Ghostscript installed','Ghostscript is necessary for nice plot export with ita_savethisplot.',3;...
    'defaultPath',pwd,'*path','Default Path','This is the path where GUIs will search for files first',3;...
    'DataPath',pwd,'path','Data Path','Use this option to set your personal working directory. Access this setting via pathStr = ita_preferences(''DataPath'')',3;...
    'ghostscriptPath','','*path','ghostscript Path','Only for windows. After Ghostscript is configured, this is 1.',3;...    
    'showChannelNames',true,'bool','Show Channelnames','Show Channelnames instead of numbers in portaudio_monitor',3;...
    'lastToolboxSetupVerNum',0,'*int','Last Toolbox Setup','internal used to see the last toolbox setup date',3;...
    'colorTableName','','*char','Name of Color Table','colortable, only ita is available right now',3;...
    %'keepmatlabmenu',true','bool','Keep MATLAB Menu', 'Keep the matlab menu additionally to the ita menu',3;... %RSC does not work yet
    'toolboxlogo',false,'bool','Logo in plots','Insert a ITA-Toolbox-Logo into your plots',3;...
    'logosize',0.2,'*int','Relative size of Logo in plots','Something between 0 and 1 can be [x y] for streching',3;...
    'logoposition',[0.01 0.01],'*int','Position of Logo in plots','Relative position of logo, [0 0] left bottom, [1 1] right top... ',3;...
    'logoalpha', 0.5 ,'*int','Logo transparency', 'Something between 0 (not transparent) and 1 (invisible)',3; ...
    'nchars', 100 ,'int','number of characters used in ita_disp', 'integer greater than 0',3; ...
    'maximizedPlot', false ,'bool_ispc','Maximized Plot', 'Fullscreen Plot under Windows',3; ...
    'license', false ,'*bool','License already accepted', '',3; ...
    'nakedClasses', false ,'bool','Show class insight', 'Disable the overloaded disp/display and show the raw classes for itaSuper, itaAudio, itaResult',3; ...
    'ColorDisp', true ,'bool','Colorful command line', 'Command line display in color (problematic on R2011b)',3; ...
    }; ...
    % IO SETTINGS Tab
    ita_preferences_playrecmidi]; % MMT: playrec preferences appear here, as they belong to the kernel now


%% search for applications preferences
pref_files = rdir([ita_toolbox_path filesep 'applications' filesep '**' filesep 'ita_preferences_*.m']);
for idx = 1:numel(pref_files)
    [dummy, filename] = fileparts(pref_files(idx).name);
    try
        res = eval(filename);
    catch errmsg
        disp('ita_defined_preferences: Some error occured. Maybe you need to do a toolbox setup.');
        disp(['ita_defined_preferences: Error in :' filename]);
        rethrow(errmsg);
    end
    defined_preferences = [defined_preferences; res]; %#ok<AGROW>
end
